@@ -107,6 +107,32 @@ void cit_backtrace(void)
 #endif
 }
 
+void cit_oneline_backtrace(void)
+{
+#ifdef HAVE_BACKTRACE
+	void *stack_frames[50];
+	size_t size, i;
+	char **strings;
+	StrBuf *Buf;
+
+	size = backtrace(stack_frames, sizeof(stack_frames) / sizeof(void*));
+	strings = backtrace_symbols(stack_frames, size);
+	if (size > 0)
+	{
+		Buf = NewStrBuf();
+		for (i = 1; i < size; i++) {
+			if (strings != NULL)
+				StrBufAppendPrintf(Buf, "%s : ", strings[i]);
+			else
+				StrBufAppendPrintf(Buf, "%p : ", stack_frames[i]);
+		}
+		free(strings);
+		syslog(LOG_ALERT, "%s\n", ChrPtr(Buf));
+		FreeStrBuf(&Buf);
+	}
+#endif
+}
+
 /*
  * print the actual stack frame.
  */
@@ -143,6 +169,7 @@ void master_startup(void) {
 	
 	syslog(LOG_DEBUG, "master_startup() started\n");
 	time(&server_startup_time);
+	get_config();
 
 	syslog(LOG_INFO, "Opening databases\n");
 	open_databases();
@@ -182,9 +209,6 @@ void master_startup(void) {
 	srand(seed);
 	srandom(seed);
 
-	syslog(LOG_INFO, "Initializing ipgm secret\n");
-	get_config();
-	config.c_ipgm_secret = rand();
 	put_config();
 
 	syslog(LOG_DEBUG, "master_startup() finished\n");
@@ -198,7 +222,7 @@ void master_cleanup(int exitcode) {
 	struct CleanupFunctionHook *fcn;
 	static int already_cleaning_up = 0;
 
-	if (already_cleaning_up) while(1) sleep(1);
+	if (already_cleaning_up) while(1) usleep(1000);
 	already_cleaning_up = 1;
 
 	/* Run any cleanup routines registered by loadable modules */
@@ -320,9 +344,9 @@ void cmd_time(char *argbuf)
    
    /* timezone and daylight global variables are not portable. */
 #ifdef HAVE_STRUCT_TM_TM_GMTOFF
-   cprintf("%d %ld|%ld|%d\n", CIT_OK, (long)tv, tmp.tm_gmtoff, tmp.tm_isdst);
+   cprintf("%d %ld|%ld|%d|%ld\n", CIT_OK, (long)tv, tmp.tm_gmtoff, tmp.tm_isdst, server_startup_time);
 #else
-   cprintf("%d %ld|%ld|%d\n", CIT_OK, (long)tv, timezone, tmp.tm_isdst);
+   cprintf("%d %ld|%ld|%d|%ld\n", CIT_OK, (long)tv, timezone, tmp.tm_isdst, server_startup_time);
 #endif
 }
 
@@ -477,7 +501,7 @@ void cmd_iden(char *argbuf)
 	cprintf("%d Ok\n",CIT_OK);
 }
 
-
+typedef const char *ccharp;
 /*
  * display system messages or help
  */
@@ -519,15 +543,15 @@ void cmd_mesg(char *mname)
 
 	/* Otherwise, look for the requested file by name. */
 	else {
-		mesg_locate(targ, sizeof targ, buf2, 2, (const char **)dirs);
+		mesg_locate(targ, sizeof targ, buf2, 2, (const ccharp*)dirs);
 		if (IsEmptyStr(targ)) {
 			snprintf(buf2, sizeof buf2, "%s.%d",
 							buf, CC->cs_clientdev);
 			mesg_locate(targ, sizeof targ, buf2, 2,
-							(const char **)dirs);
+				    (const ccharp*)dirs);
 			if (IsEmptyStr(targ)) {
 				mesg_locate(targ, sizeof targ, buf, 2,
-							(const char **)dirs);
+					    (const ccharp*)dirs);
 			}	
 		}
 	}
@@ -587,7 +611,7 @@ void cmd_emsg(char *mname)
 	dirs[0] = strdup(ctdl_message_dir);
 	dirs[1] = strdup(ctdl_hlp_dir);
 
-	mesg_locate(targ, sizeof targ, buf, 2, (const char**)dirs);
+	mesg_locate(targ, sizeof targ, buf, 2, (const ccharp*)dirs);
 	free(dirs[0]);
 	free(dirs[1]);
 
@@ -733,36 +757,6 @@ void cmd_echo(char *etext)
 }
 
 
-/* 
- * Perform privilege escalation for an internal program
- */
-void cmd_ipgm(char *argbuf)
-{
-	int secret;
-
-	secret = extract_int(argbuf, 0);
-
-	/* For security reasons, we do NOT allow this command to run
-	 * over the network.  Local sockets only.
-	 */
-	if (!CC->is_local_socket) {
-		sleep(5);
-		cprintf("%d Authentication failed.\n", ERROR + PASSWORD_REQUIRED);
-	}
-	else if (secret == config.c_ipgm_secret) {
-		CC->internal_pgm = 1;
-		strcpy(CC->curr_user, "<internal program>");
-		CC->cs_flags = CC->cs_flags|CS_STEALTH;
-		cprintf("%d Authenticated as an internal program.\n", CIT_OK);
-	}
-	else {
-		sleep(5);
-		cprintf("%d Authentication failed.\n", ERROR + PASSWORD_REQUIRED);
-		syslog(LOG_ERR, "Warning: ipgm authentication failed.\n");
-		CC->kill_me = KILLME_AUTHFAILED;
-	}
-}
-
 
 /*
  * Shut down the server
@@ -967,6 +961,12 @@ void citproto_begin_session() {
 }
 
 
+void citproto_begin_admin_session() {
+	CC->internal_pgm = 1;
+	cprintf("%d %s Citadel server ADMIN CONNECTION ready.\n", CIT_OK, config.c_nodename);
+}
+
+
 void cmd_noop(char *argbuf)
 {
 	cprintf("%d%cok\n", CIT_OK, CtdlCheckExpress() );
@@ -1081,7 +1081,6 @@ CTDL_MODULE_INIT(citserver)
 		CtdlRegisterProtoHook(cmd_echo, "ECHO", "echo text back to the client");
 		CtdlRegisterProtoHook(cmd_more, "MORE", "fetch the paginator prompt");
 		CtdlRegisterProtoHook(cmd_iden, "IDEN", "identify the client software and location");
-		CtdlRegisterProtoHook(cmd_ipgm, "IPGM", "perform privilege escalation for internal programs");
 		CtdlRegisterProtoHook(cmd_term, "TERM", "terminate another running session");
 		CtdlRegisterProtoHook(cmd_down, "DOWN", "perform a server shutdown");
 		CtdlRegisterProtoHook(cmd_halt, "HALT", "halt the server without exiting the server process");
