@@ -84,7 +84,7 @@
 #include "user_ops.h"
 #include "control.h"
 
-
+int DebugSession = 0;
 
 pthread_key_t MyConKey;				/* TSD key for MyContext() */
 
@@ -159,7 +159,7 @@ int CtdlTerminateOtherSession (int session_num)
 		return TERM_NOTALLOWED;
 	}
 
-	syslog(LOG_DEBUG, "Locating session to kill\n");
+	CONM_syslog(LOG_DEBUG, "Locating session to kill\n");
 	begin_critical_section(S_SESSION_TABLE);
 	for (ccptr = ContextList; ccptr != NULL; ccptr = ccptr->next) {
 		if (session_num == ccptr->cs_pid) {
@@ -298,9 +298,9 @@ void terminate_idle_sessions(void)
 	}
 	end_critical_section(S_SESSION_TABLE);
 	if (killed > 0)
-		syslog(LOG_INFO, "Terminated %d idle sessions", killed);
+		CON_syslog(LOG_INFO, "Scheduled %d idle sessions for termination\n", killed);
 	if (longrunners > 0)
-		syslog(LOG_INFO, "Didn't terminate %d protected idle sessions", longrunners);
+		CON_syslog(LOG_INFO, "Didn't terminate %d protected idle sessions", longrunners);
 }
 
 
@@ -316,7 +316,7 @@ void terminate_all_sessions(void)
 	for (ccptr = ContextList; ccptr != NULL; ccptr = ccptr->next) {
 		if (ccptr->client_socket != -1)
 		{
-			syslog(LOG_INFO, "terminate_all_sessions() is murdering %s", ccptr->curr_user);
+			CON_syslog(LOG_INFO, "terminate_all_sessions() is murdering %s", ccptr->curr_user);
 			close(ccptr->client_socket);
 			ccptr->client_socket = -1;
 			killed++;
@@ -324,7 +324,7 @@ void terminate_all_sessions(void)
 	}
 	end_critical_section(S_SESSION_TABLE);
 	if (killed > 0) {
-		syslog(LOG_INFO, "Flushed %d stuck sessions\n", killed);
+		CON_syslog(LOG_INFO, "Flushed %d stuck sessions\n", killed);
 	}
 }
 
@@ -337,15 +337,15 @@ void RemoveContext (CitContext *con)
 {
 	const char *c;
 	if (con == NULL) {
-		syslog(LOG_ERR, "WARNING: RemoveContext() called with NULL!");
+		CONM_syslog(LOG_ERR, "WARNING: RemoveContext() called with NULL!");
 		return;
 	}
 	c = con->ServiceName;
 	if (c == NULL) {
 		c = "WTF?";
 	}
-	syslog(LOG_DEBUG, "RemoveContext(%s) session %d", c, con->cs_pid);
-	cit_backtrace();
+	CON_syslog(LOG_DEBUG, "RemoveContext(%s) session %d", c, con->cs_pid);
+///	cit_backtrace();
 
 	/* Run any cleanup routines registered by loadable modules.
 	 * Note: We have to "become_session()" because the cleanup functions
@@ -357,7 +357,7 @@ void RemoveContext (CitContext *con)
 	client_close();				/* If the client is still connected, blow 'em away. */
 	become_session(NULL);
 
-	syslog(LOG_NOTICE, "[%3d] Session ended.", con->cs_pid);
+	CON_syslog(LOG_NOTICE, "[%3d]SRV[%s] Session ended.", con->cs_pid, c);
 
 	/* 
 	 * If the client is still connected, blow 'em away. 
@@ -365,7 +365,7 @@ void RemoveContext (CitContext *con)
 	 */
 	if (con->client_socket > 0)
 	{
-		syslog(LOG_NOTICE, "Closing socket %d", con->client_socket);
+		CON_syslog(LOG_NOTICE, "Closing socket %d", con->client_socket);
 		close(con->client_socket);
 	}
 
@@ -374,14 +374,14 @@ void RemoveContext (CitContext *con)
 		free(con->ldap_dn);
 		con->ldap_dn = NULL;
 	}
-
+	FreeStrBuf(&con->StatusMessage);
 	FreeStrBuf(&con->MigrateBuf);
-	FreeStrBuf(&con->ReadBuf);
+	FreeStrBuf(&con->RecvBuf.Buf);
 	if (con->cached_msglist) {
 		free(con->cached_msglist);
 	}
 
-	syslog(LOG_DEBUG, "Done with RemoveContext()");
+	CONM_syslog(LOG_DEBUG, "Done with RemoveContext()");
 }
 
 
@@ -398,7 +398,7 @@ CitContext *CreateNewContext(void) {
 
 	me = (CitContext *) malloc(sizeof(CitContext));
 	if (me == NULL) {
-		syslog(LOG_ALERT, "citserver: can't allocate memory!!\n");
+		CONM_syslog(LOG_ALERT, "citserver: can't allocate memory!!\n");
 		return NULL;
 	}
 	memset(me, 0, sizeof(CitContext));
@@ -416,8 +416,11 @@ CitContext *CreateNewContext(void) {
 	 * the list.
 	 */
 	me->MigrateBuf = NewStrBuf();
-	me->ReadBuf = NewStrBuf();
-	me->lastcmd = time(NULL);	/* set lastcmd to now to prevent idle timer infanticide */
+
+	me->RecvBuf.Buf = NewStrBuf();
+
+	me->lastcmd = time(NULL);	/* set lastcmd to now to prevent idle timer infanticide TODO: if we have a valid IO, use that to set the timer. */
+
 
 	begin_critical_section(S_SESSION_TABLE);
 	me->cs_pid = ++next_pid;
@@ -433,6 +436,65 @@ CitContext *CreateNewContext(void) {
 }
 
 
+/*
+ * Initialize a new context and place it in the list.  The session number
+ * used to be the PID (which is why it's called cs_pid), but that was when we
+ * had one process per session.  Now we just assign them sequentially, starting
+ * at 1 (don't change it to 0 because masterCC uses 0).
+ */
+CitContext *CloneContext(CitContext *CloneMe) {
+	CitContext *me;
+	static int next_pid = 0;
+
+	me = (CitContext *) malloc(sizeof(CitContext));
+	if (me == NULL) {
+		CONM_syslog(LOG_ALERT, "citserver: can't allocate memory!!\n");
+		return NULL;
+	}
+	memcpy(me, CloneMe, sizeof(CitContext));
+
+	memset(&me->RecvBuf, 0, sizeof(IOBuffer));
+	memset(&me->SendBuf, 0, sizeof(IOBuffer));
+	memset(&me->SBuf, 0, sizeof(IOBuffer));
+	me->MigrateBuf = NULL;
+	me->sMigrateBuf = NULL;
+	me->redirect_buffer = NULL;
+#ifdef HAVE_OPENSSL
+	me->ssl = NULL;
+#endif
+
+	me->download_fp = NULL;
+	me->upload_fp = NULL;
+	/// TODO: what about the room/user?
+	me->ma = NULL;
+	me->openid_data = NULL;
+	me->ldap_dn = NULL;
+	me->session_specific_data = NULL;
+
+	me->download_fp = NULL;
+	me->upload_fp = NULL;
+	me->client_socket = 0;
+
+	me->MigrateBuf = NewStrBuf();
+	me->RecvBuf.Buf = NewStrBuf();
+	
+	begin_critical_section(S_SESSION_TABLE);
+	{
+		me->cs_pid = ++next_pid;
+		me->prev = NULL;
+		me->next = ContextList;
+		me->lastcmd = time(NULL);	/* set lastcmd to now to prevent idle timer infanticide */
+		ContextList = me;
+		if (me->next != NULL) {
+			me->next->prev = me;
+		}
+		++num_sessions;
+	}
+	end_critical_section(S_SESSION_TABLE);
+	return (me);
+}
+
+
 /*
  * Return an array containing a copy of the context list.
  * This allows worker threads to perform "for each context" operations without
@@ -460,6 +522,44 @@ CitContext *CtdlGetContextArray(int *count)
 }
 
 
+
+/*
+ * This function fills in a context and its user field correctly
+ * Then creates/loads that user
+ */
+void CtdlFillSystemContext(CitContext *context, char *name)
+{
+	char sysname[SIZ];
+	long len;
+
+	memset(context, 0, sizeof(CitContext));
+	context->internal_pgm = 1;
+	context->cs_pid = 0;
+	strcpy (sysname, "SYS_");
+	strcat (sysname, name);
+	len = cutuserkey(sysname);
+	memcpy(context->curr_user, sysname, len + 1);
+	context->client_socket = (-1);
+	context->state = CON_SYS;
+	context->ServiceName = name;
+
+	/* internal_create_user has the side effect of loading the user regardless of wether they
+	 * already existed or needed to be created
+	 */
+	internal_create_user (sysname, len, &(context->user), -1) ;
+	
+	/* Check to see if the system user needs upgrading */
+	if (context->user.usernum == 0)
+	{	/* old system user with number 0, upgrade it */
+		context->user.usernum = get_new_user_number();
+		CON_syslog(LOG_INFO, "Upgrading system user \"%s\" from user number 0 to user number %ld\n", context->user.fullname, context->user.usernum);
+		/* add user to the database */
+		CtdlPutUser(&(context->user));
+		cdb_store(CDB_USERSBYNUMBER, &(context->user.usernum), sizeof(long), context->user.fullname, strlen(context->user.fullname)+1);
+	}
+}
+
+
 /*
  * Cleanup any contexts that are left lying around
  */
@@ -485,8 +585,8 @@ void context_cleanup(void)
 		/* Remove the session from the active list */
 		rem = ptr->next;
 		--num_sessions;
-		
-		syslog(LOG_DEBUG, "context_cleanup(): purging session %d\n", ptr->cs_pid);
+
+		CON_syslog(LOG_DEBUG, "context_cleanup(): purging session %d\n", ptr->cs_pid);
 		RemoveContext(ptr);
 		free (ptr);
 		ptr = rem;
@@ -546,7 +646,7 @@ void dead_session_purge(int force) {
 	 * is allocated privately on this thread's stack.
 	 */
 	while (rem != NULL) {
-		syslog(LOG_DEBUG, "dead_session_purge(): purging session %d, reason=%d\n", rem->cs_pid, rem->kill_me);
+		CON_syslog(LOG_DEBUG, "dead_session_purge(): purging session %d, reason=%d\n", rem->cs_pid, rem->kill_me);
 		RemoveContext(rem);
 		ptr = rem;
 		rem = rem->next;
@@ -577,7 +677,7 @@ void InitializeMasterCC(void) {
  */
 void set_async_waiting(struct CitContext *ccptr)
 {
-	syslog(LOG_DEBUG, "Setting async_waiting flag for session %d\n", ccptr->cs_pid);
+	CON_syslog(LOG_DEBUG, "Setting async_waiting flag for session %d\n", ccptr->cs_pid);
 	if (ccptr->is_async) {
 		ccptr->async_waiting++;
 		if (ccptr->state == CON_IDLE) {
@@ -585,3 +685,16 @@ void set_async_waiting(struct CitContext *ccptr)
 		}
 	}
 }
+
+
+void DebugSessionEnable(const int n)
+{
+	DebugSession = n;
+}
+CTDL_MODULE_INIT(session)
+{
+	if (!threading) {
+		CtdlRegisterDebugFlagHook(HKEY("session"), DebugSessionEnable, &DebugSession);
+	}
+	return "session";
+}
