@@ -309,7 +309,7 @@ void FreeAsyncIOContents(AsyncIO *IO)
 }
 
 
-void StopClientWatchers(AsyncIO *IO)
+void StopClientWatchers(AsyncIO *IO, int CloseFD)
 {
 	ev_timer_stop (event_base, &IO->rw_timeout);
 	ev_timer_stop(event_base, &IO->conn_fail);
@@ -320,11 +320,11 @@ void StopClientWatchers(AsyncIO *IO)
 	ev_io_stop(event_base, &IO->send_event);
 	ev_io_stop(event_base, &IO->recv_event);
 
-	if (IO->SendBuf.fd != 0) {
+	if (CloseFD && (IO->SendBuf.fd > 0)) {
 		close(IO->SendBuf.fd);
+		IO->SendBuf.fd = -1;
+		IO->RecvBuf.fd = -1;
 	}
-	IO->SendBuf.fd = 0;
-	IO->RecvBuf.fd = 0;
 }
 
 void StopCurlWatchers(AsyncIO *IO)
@@ -352,7 +352,7 @@ void ShutDownCLient(AsyncIO *IO)
 
 	EVM_syslog(LOG_DEBUG, "EVENT Terminating \n");
 
-	StopClientWatchers(IO);
+	StopClientWatchers(IO, 1);
 
 	if (IO->DNS.Channel != NULL) {
 		ares_destroy(IO->DNS.Channel);
@@ -495,8 +495,8 @@ IO_send_callback(struct ev_loop *loop, ev_io *watcher, int revents)
 				StrBufPlain(IO->ErrMsg, errmsg, -1);
 			break;
 		default:
-			rc = StrBuf_write_one_chunk_callback(watcher->fd,
-							     0/*TODO*/,
+			rc = StrBuf_write_one_chunk_callback(IO->SendBuf.fd,
+							     0,
 							     &IO->SendBuf);
 		}
 
@@ -584,13 +584,25 @@ IO_send_callback(struct ev_loop *loop, ev_io *watcher, int revents)
 		}
 	}
 	else if (rc < 0) {
-		IO_Timeout_callback(loop, &IO->rw_timeout, revents);
+		if (errno != EAGAIN) {
+			StopClientWatchers(IO, 1);
+			EV_syslog(LOG_DEBUG,
+				  "EVENT: Socket Invalid! [%d] [%s] [%d]\n",
+				  errno, strerror(errno), IO->SendBuf.fd);
+			StrBufPrintf(IO->ErrMsg,
+				     "Socket Invalid! [%s]",
+				     strerror(errno));
+			SetNextTimeout(IO, 0.0);
+		}
 	}
 	/* else : must write more. */
 }
 static void
 set_start_callback(struct ev_loop *loop, AsyncIO *IO, int revents)
 {
+	ev_timer_stop(event_base, &IO->conn_fail);
+	ev_timer_start(event_base, &IO->rw_timeout);
+
 	switch(IO->NextState) {
 	case eReadMore:
 	case eReadMessage:
@@ -722,7 +734,7 @@ IO_connestd_callback(struct ev_loop *loop, ev_io *watcher, int revents)
                          (void*)&so_err,
                          &lon);
 
-        if ((err == 0) && (so_err == 111))
+        if ((err == 0) && (so_err != 0))
         {
                 EV_syslog(LOG_DEBUG, "connect() failed [%d][%s]\n",
                           so_err,
@@ -765,8 +777,8 @@ IO_recv_callback(struct ev_loop *loop, ev_io *watcher, int revents)
 		}
 		break;
 	default:
-		nbytes = StrBuf_read_one_chunk_callback(watcher->fd,
-							0 /*TODO */,
+		nbytes = StrBuf_read_one_chunk_callback(IO->RecvBuf.fd,
+							0,
 							&IO->RecvBuf);
 		break;
 	}
@@ -800,15 +812,20 @@ IO_recv_callback(struct ev_loop *loop, ev_io *watcher, int revents)
 	if (nbytes > 0) {
 		HandleInbound(IO);
 	} else if (nbytes == 0) {
-		IO_Timeout_callback(loop, &IO->rw_timeout, revents);
+		SetNextTimeout(IO, 0.0);
 		return;
 	} else if (nbytes == -1) {
-		// FD is gone. kick it. 
-		StopClientWatchers(IO);
-		EV_syslog(LOG_DEBUG,
-			  "EVENT: Socket Invalid! %s \n",
-			  strerror(errno));
-		ShutDownCLient(IO);
+		if (errno != EAGAIN) {
+			// FD is gone. kick it. 
+			StopClientWatchers(IO, 1);
+			EV_syslog(LOG_DEBUG,
+				  "EVENT: Socket Invalid! [%d] [%s] [%d]\n",
+				  errno, strerror(errno), IO->SendBuf.fd);
+			StrBufPrintf(IO->ErrMsg,
+				     "Socket Invalid! [%s]",
+				     strerror(errno));
+			SetNextTimeout(IO, 0.0);
+		}
 		return;
 	}
 }
@@ -948,7 +965,6 @@ eNextState EvConnectSock(AsyncIO *IO,
 	if (rc >= 0){
 		EVM_syslog(LOG_DEBUG, "connect() immediate success.\n");
 		set_start_callback(event_base, IO, 0);
-		ev_timer_start(event_base, &IO->rw_timeout);
 		return IO->NextState;
 	}
 	else if (errno == EINPROGRESS) {
