@@ -87,6 +87,7 @@ extern FixedOutputHook *FixedOutputTable;
 typedef struct SessionFunctionHook SessionFunctionHook;
 struct SessionFunctionHook {
 	SessionFunctionHook *next;
+	int Priority;
 	void (*h_function_pointer) (void);
 	int eventtype;
 };
@@ -493,20 +494,29 @@ void CtdlRegisterCleanupHook(void (*fcn_ptr) (void))
 
 void CtdlUnregisterCleanupHook(void (*fcn_ptr) (void))
 {
-	CleanupFunctionHook *cur, *p;
-
-	for (cur = CleanupHookTable; cur != NULL; cur = cur->next) {
-		/* This will also remove duplicates if any */
-		while (cur != NULL &&
-				fcn_ptr == cur->h_function_pointer) {
+	CleanupFunctionHook *cur, *p, *last;
+	last = NULL;
+	cur = CleanupHookTable;
+	while (cur != NULL)
+	{
+		if (fcn_ptr == cur->h_function_pointer)
+		{
 			MODM_syslog(LOG_DEBUG, "Unregistered cleanup function\n");
 			p = cur->next;
-			if (cur == CleanupHookTable) {
-				CleanupHookTable = p;
-			}
+
 			free(cur);
+			cur = NULL;
+
+			if (last != NULL)
+				last->next = p;
+			else 
+				CleanupHookTable = p;
 			cur = p;
 		}
+		else {
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 
@@ -543,20 +553,29 @@ void CtdlRegisterEVCleanupHook(void (*fcn_ptr) (void))
 
 void CtdlUnregisterEVCleanupHook(void (*fcn_ptr) (void))
 {
-	CleanupFunctionHook *cur, *p;
-
-	for (cur = EVCleanupHookTable; cur != NULL; cur = cur->next) {
-		/* This will also remove duplicates if any */
-		while (cur != NULL &&
-				fcn_ptr == cur->h_function_pointer) {
+	CleanupFunctionHook *cur, *p, *last;
+	last = NULL;
+	cur = EVCleanupHookTable;
+	while (cur != NULL)
+	{
+		if (fcn_ptr == cur->h_function_pointer)
+		{
 			MODM_syslog(LOG_DEBUG, "Unregistered cleanup function\n");
 			p = cur->next;
-			if (cur == EVCleanupHookTable) {
-				EVCleanupHookTable = p;
-			}
+
 			free(cur);
+			cur = NULL;
+
+			if (last != NULL)
+				last->next = p;
+			else 
+				EVCleanupHookTable = p;
 			cur = p;
 		}
+		else {
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 
@@ -577,41 +596,57 @@ void CtdlDestroyEVCleanupHooks(void)
 }
 
 
-void CtdlRegisterSessionHook(void (*fcn_ptr) (void), int EventType)
+void CtdlRegisterSessionHook(void (*fcn_ptr) (void), int EventType, int Priority)
 {
-
 	SessionFunctionHook *newfcn;
 
 	newfcn = (SessionFunctionHook *)
 	    malloc(sizeof(SessionFunctionHook));
-	newfcn->next = SessionHookTable;
+	newfcn->Priority = Priority;
 	newfcn->h_function_pointer = fcn_ptr;
 	newfcn->eventtype = EventType;
-	SessionHookTable = newfcn;
 
-	MOD_syslog(LOG_DEBUG, "Registered a new session function (type %d)\n",
-		   EventType);
+	SessionFunctionHook **pfcn;
+	pfcn = &SessionHookTable;
+	while ((*pfcn != NULL) && 
+	       ((*pfcn)->Priority < newfcn->Priority) &&
+	       ((*pfcn)->next != NULL))
+		pfcn = &(*pfcn)->next;
+		
+	newfcn->next = *pfcn;
+	*pfcn = newfcn;
+	
+	MOD_syslog(LOG_DEBUG, "Registered a new session function (type %d Priority %d)\n",
+		   EventType, Priority);
 }
 
 
 void CtdlUnregisterSessionHook(void (*fcn_ptr) (void), int EventType)
 {
-	SessionFunctionHook *cur, *p;
-
-	for (cur = SessionHookTable; cur != NULL; cur = cur->next) {
-		/* This will also remove duplicates if any */
-		while (cur != NULL &&
-				fcn_ptr == cur->h_function_pointer &&
-				EventType == cur->eventtype) {
+	SessionFunctionHook *cur, *p, *last;
+	last = NULL;
+	cur = SessionHookTable;
+	while  (cur != NULL) {
+		if ((fcn_ptr == cur->h_function_pointer) &&
+		    (EventType == cur->eventtype))
+		{
 			MOD_syslog(LOG_DEBUG, "Unregistered session function (type %d)\n",
 				   EventType);
 			p = cur->next;
-			if (cur == SessionHookTable) {
-				SessionHookTable = p;
-			}
+
 			free(cur);
+			cur = NULL;
+
+			if (last != NULL)
+				last->next = p;
+			else 
+				SessionHookTable = p;
 			cur = p;
 		}
+		else {
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 
@@ -650,22 +685,30 @@ void CtdlRegisterUserHook(void (*fcn_ptr) (ctdluser *), int EventType)
 
 void CtdlUnregisterUserHook(void (*fcn_ptr) (struct ctdluser *), int EventType)
 {
-	UserFunctionHook *cur, *p;
-
-	for (cur = UserHookTable; cur != NULL; cur = cur->next) {
-		/* This will also remove duplicates if any */
-		while (cur != NULL &&
-				fcn_ptr == cur->h_function_pointer &&
-				EventType == cur->eventtype) {
+	UserFunctionHook *cur, *p, *last;
+	last = NULL;
+	cur = UserHookTable;
+	while (cur != NULL) {
+		if ((fcn_ptr == cur->h_function_pointer) &&
+		    (EventType == cur->eventtype))
+		{
 			MOD_syslog(LOG_DEBUG, "Unregistered user function (type %d)\n",
 				   EventType);
 			p = cur->next;
-			if (cur == UserHookTable) {
-				UserHookTable = p;
-			}
+
 			free(cur);
+			cur = NULL;
+
+			if (last != NULL)
+				last->next = p;
+			else 
+				UserHookTable = p;
 			cur = p;
 		}
+		else {
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 
@@ -706,22 +749,29 @@ void CtdlRegisterMessageHook(int (*handler)(struct CtdlMessage *),
 void CtdlUnregisterMessageHook(int (*handler)(struct CtdlMessage *),
 		int EventType)
 {
-	MessageFunctionHook *cur, *p;
-
-	for (cur = MessageHookTable; cur != NULL; cur = cur->next) {
-		/* This will also remove duplicates if any */
-		while (cur != NULL &&
-				handler == cur->h_function_pointer &&
-				EventType == cur->eventtype) {
+	MessageFunctionHook *cur, *p, *last;
+	last = NULL;
+	cur = MessageHookTable;
+	while (cur != NULL) {
+		if ((handler == cur->h_function_pointer) &&
+		    (EventType == cur->eventtype))
+		{
 			MOD_syslog(LOG_DEBUG, "Unregistered message function (type %d)\n",
 				   EventType);
 			p = cur->next;
-			if (cur == MessageHookTable) {
-				MessageHookTable = p;
-			}
 			free(cur);
+			cur = NULL;
+
+			if (last != NULL)
+				last->next = p;
+			else 
+				MessageHookTable = p;
 			cur = p;
 		}
+		else {
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 
@@ -757,18 +807,28 @@ void CtdlRegisterRoomHook(int (*fcn_ptr)(struct ctdlroom *))
 
 void CtdlUnregisterRoomHook(int (*fcn_ptr)(struct ctdlroom *))
 {
-	RoomFunctionHook *cur, *p;
-
-	for (cur = RoomHookTable; cur != NULL; cur = cur->next) {
-		while (cur != NULL && fcn_ptr == cur->fcn_ptr) {
+	RoomFunctionHook *cur, *p, *last;
+	last = NULL;
+	cur = RoomHookTable;
+	while (cur != NULL)
+	{
+		if (fcn_ptr == cur->fcn_ptr) {
 			MODM_syslog(LOG_DEBUG, "Unregistered room function\n");
 			p = cur->next;
-			if (cur == RoomHookTable) {
-				RoomHookTable = p;
-			}
+
 			free(cur);
+			cur = NULL;
+
+			if (last != NULL)
+				last->next = p;
+			else 
+				RoomHookTable = p;
 			cur = p;
 		}
+		else {
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 
@@ -804,20 +864,29 @@ void CtdlRegisterNetprocHook(int (*handler)(struct CtdlMessage *, char *) )
 
 void CtdlUnregisterNetprocHook(int (*handler)(struct CtdlMessage *, char *) )
 {
-	NetprocFunctionHook *cur, *p;
+	NetprocFunctionHook *cur, *p, *last;
 
-	for (cur = NetprocHookTable; cur != NULL; cur = cur->next) {
-		/* This will also remove duplicates if any */
-		while (cur != NULL &&
-				handler == cur->h_function_pointer ) {
+	cur = NetprocHookTable;
+	last = NULL;
+
+	while (cur != NULL) {
+		if (handler == cur->h_function_pointer)
+		{
 			MODM_syslog(LOG_DEBUG, "Unregistered netproc function\n");
 			p = cur->next;
-			if (cur == NetprocHookTable) {
+			free(cur);
+			if (last != NULL) {
+				last->next = p;
+			}
+			else {
 				NetprocHookTable = p;
 			}
-			free(cur);
 			cur = p;
 		}
+		else {
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 
@@ -853,20 +922,28 @@ void CtdlRegisterDeleteHook(void (*handler)(char *, long) )
 
 void CtdlUnregisterDeleteHook(void (*handler)(char *, long) )
 {
-	DeleteFunctionHook *cur, *p;
+	DeleteFunctionHook *cur, *p, *last;
 
-	for (cur = DeleteHookTable; cur != NULL; cur = cur->next) {
-		/* This will also remove duplicates if any */
-		while (cur != NULL &&
-				handler == cur->h_function_pointer ) {
+	last = NULL;
+	cur = DeleteHookTable;
+	while (cur != NULL) {
+		if (handler == cur->h_function_pointer )
+		{
 			MODM_syslog(LOG_DEBUG, "Unregistered delete function\n");
 			p = cur->next;
-			if (cur == DeleteHookTable) {
-				DeleteHookTable = p;
-			}
 			free(cur);
+
+			if (last != NULL)
+				last->next = p;
+			else
+				DeleteHookTable = p;
+
 			cur = p;
 		}
+		else {
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 void CtdlDestroyDeleteHooks(void)
@@ -904,19 +981,32 @@ void CtdlRegisterFixedOutputHook(char *content_type, void (*handler)(char *, int
 
 void CtdlUnregisterFixedOutputHook(char *content_type)
 {
-	FixedOutputHook *cur, *p;
+	FixedOutputHook *cur, *p, *last;
 
-	for (cur = FixedOutputTable; cur != NULL; cur = cur->next) {
+	last = NULL;
+	cur = FixedOutputTable;
+	while (cur != NULL) {
 		/* This will also remove duplicates if any */
-		while (cur != NULL && (!strcasecmp(content_type, cur->content_type))) {
-			MOD_syslog(LOG_DEBUG, "Unregistered fixed output function for %s\n", content_type);
+		if (!strcasecmp(content_type, cur->content_type)) {
+			MOD_syslog(LOG_DEBUG,
+				   "Unregistered fixed output function for %s\n",
+				   content_type);
+
 			p = cur->next;
-			if (cur == FixedOutputTable) {
-				FixedOutputTable = p;
-			}
 			free(cur);
+
+			if (last != NULL)
+				last->next = p;
+			else
+				FixedOutputTable = p;
+			
 			cur = p;
 		}
+		else
+		{
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 
@@ -970,22 +1060,30 @@ void CtdlRegisterXmsgHook(int (*fcn_ptr) (char *, char *, char *, char *), int o
 
 void CtdlUnregisterXmsgHook(int (*fcn_ptr) (char *, char *, char *, char *), int order)
 {
-	XmsgFunctionHook *cur, *p;
+	XmsgFunctionHook *cur, *p, *last;
 
-	for (cur = XmsgHookTable; cur != NULL; cur = cur->next) {
+	last = NULL;
+	cur = XmsgHookTable;
+	while (cur != NULL) {
 		/* This will also remove duplicates if any */
-		while (cur != NULL &&
-				fcn_ptr == cur->h_function_pointer &&
-				order == cur->order) {
+		if (fcn_ptr == cur->h_function_pointer &&
+		    order == cur->order) {
 			MOD_syslog(LOG_DEBUG, "Unregistered x-msg function "
 				   "(priority %d)\n", order);
 			p = cur->next;
-			if (cur == XmsgHookTable) {
-				XmsgHookTable = p;
-			}
 			free(cur);
+
+			if (last != NULL)
+				last->next = p;
+			else
+				XmsgHookTable = p;
+			
 			cur = p;
 		}
+		else {
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 
@@ -1070,19 +1168,20 @@ void CtdlUnregisterServiceHook(int tcp_port, char *sockpath,
 			void (*h_async_function) (void)
 			)
 {
-	ServiceFunctionHook *cur, *p;
+	ServiceFunctionHook *cur, *p, *last;
 
+	last = NULL;
 	cur = ServiceHookTable;
 	while (cur != NULL) {
 		/* This will also remove duplicates if any */
-		while (cur != NULL &&
-				!(sockpath && cur->sockpath &&
-					strcmp(sockpath, cur->sockpath)) &&
-				h_greeting_function == cur->h_greeting_function &&
-				h_command_function == cur->h_command_function &&
-				h_async_function == cur->h_async_function &&
-				tcp_port == cur->tcp_port) {
-			close(cur->msock);
+		if (h_greeting_function == cur->h_greeting_function &&
+		    h_command_function == cur->h_command_function &&
+		    h_async_function == cur->h_async_function &&
+		    tcp_port == cur->tcp_port && 
+		    !(sockpath && cur->sockpath && strcmp(sockpath, cur->sockpath)) )
+		{
+			if (cur->msock > 0)
+				close(cur->msock);
 			if (sockpath) {
 				MOD_syslog(LOG_INFO, "Closed UNIX domain socket %s\n",
 					   sockpath);
@@ -1093,12 +1192,17 @@ void CtdlUnregisterServiceHook(int tcp_port, char *sockpath,
 				MOD_syslog(LOG_INFO, "Unregistered service \"%s\"\n", cur->ServiceName);
 			}
 			p = cur->next;
-			if (cur == ServiceHookTable) {
-				ServiceHookTable = p;
-			}
 			free(cur);
+			if (last != NULL)
+				last->next = p;
+			else
+				ServiceHookTable = p;
 			cur = p;
 		}
+		else {
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 
@@ -1173,18 +1277,28 @@ void CtdlRegisterSearchFuncHook(void (*fcn_ptr)(int *, long **, const char *), c
 
 void CtdlUnregisterSearchFuncHook(void (*fcn_ptr)(int *, long **, const char *), char *name)
 {
-	SearchFunctionHook *cur, *p;
+	SearchFunctionHook *cur, *p, *last;
 	
-	for (cur = SearchFunctionHookTable; cur != NULL; cur = cur->next) {
-		while (fcn_ptr && (cur->fcn_ptr == fcn_ptr) && name && !strcmp(name, cur->name)) {
+	last = NULL;
+	cur = SearchFunctionHookTable;
+	while (cur != NULL) {
+		if (fcn_ptr &&
+		    (cur->fcn_ptr == fcn_ptr) &&
+		    name && !strcmp(name, cur->name))
+		{
 			MOD_syslog(LOG_DEBUG, "Unregistered search function(%s)\n", name);
 			p = cur->next;
-			if (cur == SearchFunctionHookTable) {
-				SearchFunctionHookTable = p;
-			}
 			free (cur);
+			if (last != NULL)
+				last->next = p;
+			else
+				SearchFunctionHookTable = p;
 			cur = p;
 		}
+		else {
+			last = cur;
+			cur = cur->next;
+		}
 	}
 }
 
